<?php

namespace Drupal\module_timer\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a controller for handling Module Timer functionality.
 */
class ModuleTimerController extends ControllerBase {

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs the ModuleTimerController.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service, used for displaying system messages to users.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessengerInterface $messenger, DateFormatterInterface $date_formatter, FormBuilderInterface $form_builder) {
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
    $this->dateFormatter = $date_formatter;
    $this->formBuilder = $form_builder;
  }

  /**
   * Dependency injection for the controller.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container, used for fetching dependencies.
   *
   * @return static
   *   An instance of the ModuleTimerController with dependencies injected.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('form_builder')
    );
  }

  /**
   * Provides the content for the module timer overview page.
   *
   * @return array
   *   A render array containing the module timer table and add timer button.
   */
  public function content() {
    // Load configuration.
    $config = $this->configFactory->get('module_timer.settings');
    $module_timers = $config->get('module_timers') ?? [];

    // Define sortable headers.
    $header = [
      ['data' => $this->t('Updated Time'), 'field' => 'updated_time'],
      ['data' => $this->t('Module Names'), 'field' => 'module_names'],
      ['data' => $this->t('Duration'), 'field' => 'duration'],
      $this->t('Operations'),
    ];

    $rows = [];
    foreach ($module_timers as $key => $timer) {

      // Add the updated_time as a formatted date in the table row.
      $updated_time = isset($timer['updated_time'])
            ? $this->dateFormatter->format($timer['updated_time'], 'custom', 'Y-m-d H:i:s')
            : $this->t('N/A');

      // Join all module names into a single string separated by commas.
      $module_names = !empty($timer['module_names']) ? implode(', ', $timer['module_names']) : $this->t('None');

      $row = [
        $updated_time,
        // Display the concatenated module names.
        $module_names,
        $timer['duration'] . ' minutes',
        [
          'data' => [
            '#type' => 'operations',
            '#links' => [
              'edit' => [
                'title' => $this->t('Edit'),
                'url' => Url::fromRoute('module_timer.edit_form', ['key' => $key]),
              ],
              'delete' => [
                'title' => $this->t('Delete'),
                'url' => Url::fromRoute('module_timer.delete', ['key' => $key]),
              ],
            ],
          ],
        ],
      ];
      $rows[] = $row;
    }

    // Add button.
    $build['add_timer'] = [
      '#type' => 'link',
      '#title' => $this->t('Add New Timer'),
      '#url' => Url::fromRoute('module_timer.add_form'),
      '#attributes' => ['class' => ['button', 'button--primary']],
    ];

    // Table.
    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No timers configured.'),
    ];

    return $build;
  }

  /**
   * Deletes a timer by key.
   */
  public function delete($key) {
    // Load the confirmation form dynamically with the key.
    return $this->formBuilder->getForm('\Drupal\module_timer\Form\ModuleTimerDeleteForm', $key);
  }

}
