<?php

namespace Drupal\module_timer\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a confirmation form before deleting a timer.
 */
class ModuleTimerDeleteForm extends ConfirmFormBase {

  /**
   * The key of the timer to delete.
   *
   * @var int
   */
  protected $key;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs the form.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service, used for displaying system messages to the user.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessengerInterface $messenger, RouteMatchInterface $route_match) {
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
    $this->routeMatch = $route_match;
  }

  /**
   * Dependency injection for the form.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container, which provides access to all registered services.
   *
   * @return static
   *   An instance of the form class with all dependencies injected.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'module_timer_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $key = NULL) {
    // Retrieve the key from the route parameters.
    $this->key = $this->routeMatch->getParameter('key');

    // Add a hidden field to store the key in the form.
    $form['key'] = [
      '#type' => 'hidden',
      '#value' => $this->key,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete this timer?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('module_timer.list');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Load configuration and delete the timer by key.
    $config = $this->configFactory->getEditable('module_timer.settings');
    $timers = $config->get('module_timers') ?? [];

    if (isset($timers[$this->key])) {
      unset($timers[$this->key]);
      $config->set('module_timers', array_values($timers))->save();
      $this->messenger->addMessage($this->t('The timer has been deleted successfully.'));
    }
    else {
      $this->messenger->addError($this->t('The timer could not be found.'));
    }

    // Redirect to the list page.
    $form_state->setRedirect('module_timer.list');
  }

}
