<?php

namespace Drupal\module_timer\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Defines the Module Timer configuration form.
 */
class ModuleTimerForm extends ConfigFormBase {

  /**
   * The key associated with the form instance.
   *
   * @var string|null
   */
  public $key;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The module extension list service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The module installer service.
   *
   * @var \Drupal\Core\Extension\ModuleInstallerInterface
   */
  protected $moduleInstaller;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs the ModuleTimerForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $module_extension_list
   *   The module extension list service.
   * @param \Drupal\Core\Extension\ModuleInstallerInterface $module_installer
   *   The module installer service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    RequestStack $request_stack,
    ModuleHandlerInterface $module_handler,
    ModuleExtensionList $module_extension_list,
    ModuleInstallerInterface $module_installer,
    TimeInterface $time
  ) {
    parent::__construct($config_factory);
    $this->requestStack = $request_stack;
    $this->moduleHandler = $module_handler;
    $this->moduleExtensionList = $module_extension_list;
    $this->moduleInstaller = $module_installer;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('module_handler'),
      $container->get('extension.list.module'),
      $container->get('module_installer'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'module_timer_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['module_timer.settings'];
  }

  /**
   * Builds the configuration form structure.
   *
   * @param array $form
   *   An associative array containing the initial structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string|null $key
   *   (optional) A key associated with this form instance, if applicable.
   *
   * @return array
   *   A renderable array representing the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $key = NULL) {
    // Fetch configuration and timers.
    $config = $this->config('module_timer.settings');
    $timers = $config->get('module_timers') ?? [];

    $this->key = $key;

    // If we are editing existing timer and fetch its details or set defaults.
    $timer = isset($key) && isset($timers[$key]) ? $timers[$key] : [
      'module_names' => [],
      'duration' => '',
    ];

    // Fetch the 'modules' parameter from the URL, if present.
    $modules_param = $this->requestStack->getCurrentRequest()->query->get('modules');
    $modules = $modules_param ? unserialize(data: base64_decode($modules_param), options: ['allowed_classes' => FALSE]) : NULL;

    // Generate the module options.
    $module_handler = $this->moduleHandler;
    $all_modules = $this->moduleExtensionList->getList();

    $module_options = [];
    foreach ($all_modules as $module => $extension) {
      $name = $extension->info['name'];
      $status = $module_handler->moduleExists($module) ? $this->t('Installed') : $this->t('Uninstalled');
      $module_options[$module] = "{$name} ({$module} - {$status})";
    }

    // Set selected modules either from existing configuration or new input.
    $selected_modules = !empty($modules) ? array_keys($modules['install']) : $timer['module_names'];

    $form['module_names'] = [
      '#type' => 'select2',
      '#title' => $this->t('Module names'),
      '#options' => $module_options,
      '#default_value' => $selected_modules,
      '#description' => $this->t('Select the modules to enable.'),
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#attributes' => ['class' => ['select2']],
    ];

    $form['duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Duration'),
      '#default_value' => $timer['duration'],
      '#description' => $this->t('Duration in minutes that the module will remain active.'),
      '#min' => 1,
      '#required' => TRUE,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    // Display dependencies message if any.
    if ($modules && isset($modules['dependencies'])) {
      foreach ($modules['dependencies'] as $module => $deps) {
        if (!empty($deps)) {
          $dependency_list = [];
          foreach ($deps as $dep => $dep_name) {
            $dependency_list[] = "$dep ($dep_name)";
          }
          $dependencies_msg = $this->t(
              '@module requires the following dependencies: @dependencies', [
                '@module' => $module,
                '@dependencies' => implode(', ', $dependency_list),
              ]
            );
          $this->messenger()->addMessage($dependencies_msg);
        }
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $key = $this->key;
    $module_names = $form_state->getValue('module_names');
    if (empty($module_names)) {
      $form_state->setErrorByName('module_names', $this->t('Please select at least one module.'));
    }

    $config = $this->config('module_timer.settings');
    $timers = $config->get('module_timers') ?? [];
    $all_used_modules = [];
    foreach ($timers as $index => $timer) {
      if ($index !== $key) {
        $all_used_modules = array_merge($all_used_modules, $timer['module_names']);
      }
    }

    $duplicate_modules = array_intersect($module_names, $all_used_modules);
    if (!empty($duplicate_modules) && count($duplicate_modules) >= 1 && $key == NULL) {
      $form_state->setErrorByName('module_names', $this->t('The following modules are already in use in another timer: @modules', ['@modules' => implode(', ', $duplicate_modules)]));
    }
  }

  /**
   * Handles the form submission.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   An object containing the current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve submitted values.
    $key = $this->key;
    $module_names = $form_state->getValue('module_names');
    $duration = $form_state->getValue('duration');

    // Load configuration and timers.
    $config = $this->config('module_timer.settings');
    $timers = $config->get('module_timers') ?? [];

    // Get the module handler and installer services.
    $module_handler = $this->moduleHandler;
    $module_installer = $this->moduleInstaller;

    // Track installed modules for messaging.
    $installed_modules = [];

    // Install each module if not already installed.
    foreach ($module_names as $module_name) {
      if (!$module_handler->moduleExists($module_name)) {
        try {
          $module_installer->install([$module_name], TRUE);
          $installed_modules[] = $module_name;
        }
        catch (\Exception $e) {
          $this->messenger()->addError(
            $this->t(
              'Failed to install module @module: @error', [
                '@module' => $module_name,
                '@error' => $e->getMessage(),
              ]
            )
          );
          // Stop further processing if installation fails.
          return;
        }
      }
    }

    // Get the current time for updated_time.
    $current_time = $this->time->getCurrentTime();

    // Prepare timer data.
    $timer_data = [
      'module_names' => array_values($module_names),
      'duration' => $duration,
      'updated_time' => $current_time,
    ];

    // Add new timer or update existing one.
    if ($key === NULL || $key === '') {
      $timers[] = $timer_data;
    }
    else {
      $timers[$key] = $timer_data;
    }

    // Save updated timers configuration.
    $config->set('module_timers', $timers)->save();

    // Create success messages for installations and timer save.
    if (!empty($installed_modules)) {
      $this->messenger()->addMessage($this->t('The following modules have been installed: @modules.', ['@modules' => implode(', ', $installed_modules)]));
    }
    $this->messenger()->addMessage($this->t('Timer has been saved successfully.'));

    // Redirect to the list.
    $form_state->setRedirect('module_timer.list');
  }

}
